import React, { useState, useEffect } from "react";
import { ReactCompareSlider } from "react-compare-slider";
import { getImageString } from "../helpers/calls";
import ViewportWrapper from "./viewHelpers/ViewportWrapper";

const state = {
  tools: [
    // Mouse
    {
      name: "Wwwc",
      mode: "active",
      modeOptions: { mouseButtonMask: 1 },
    },
    {
      name: "Zoom",
      mode: "active",
      modeOptions: { mouseButtonMask: 2 },
    },
    {
      name: "Pan",
      mode: "active",
      modeOptions: { mouseButtonMask: 4 },
    },
    // Scroll
    { name: "StackScrollMouseWheel", mode: "disabled" },
    // Touch
    { name: "PanMultiTouch", mode: "active" },
    { name: "ZoomTouchPinch", mode: "active" },
    { name: "StackScrollMultiTouch", mode: "active" },
  ],

  cornerstoneElement: undefined,
};

function formatSeries(series) {
  return series.map((i) => getImageString(i));
}

export default function Viewer({
  imageSeriesLeft,
  imageSeriesRight,
  splitWindow,
  colormapLeft,
  colormapRight,
  inverted,
  opacity,
  currentFilter,
  sync,
}) {
  // use effect that runs every time the imageSeries changes
  const [formattedImageSeriesLeft, setFormattedImageSeriesLeft] =
    React.useState([]);
  const [formattedImageSeriesRight, setFormattedImageSeriesRight] =
    React.useState([]);
  const [loading, setLoading] = useState(true);

  useEffect(() => {
    setFormattedImageSeriesLeft(formatSeries(imageSeriesLeft));
    setFormattedImageSeriesRight(formatSeries(imageSeriesRight));
    setLoading(false);
  }, [imageSeriesLeft, imageSeriesRight]);

  const [imageIndexLeft, setImageIndexLeft] = useState(0);
  const [imageIndexRight, setImageIndexRight] = useState(0);
  const [lengthLeft, setLenghtLeft] = useState(0);
  const [lengthRight, setLenghtRight] = useState(0);

  function handleChangeLeft(event) {
    const value = event.target.value;
    const delta = parseInt(value) - parseInt(imageIndexLeft);
    setImageIndexLeft(value);
    if (sync) {
      setImageIndexRight(parseInt(imageIndexRight) + parseInt(delta));
    }
  }

  function handleChangeRight(event) {
    const value = event.target.value;
    const delta = parseInt(value) - parseInt(imageIndexRight);
    setImageIndexRight(value);
    if (sync) {
      setImageIndexLeft(parseInt(imageIndexLeft) + parseInt(delta));
    }
  }

  // Eventlistener for scrollwheel to change imageIndex
  useEffect(() => {
    const handleWheel = (e) => {
      if (e.deltaY > 0) {
        if (
          imageIndexLeft < formattedImageSeriesLeft.length - 1 &&
          imageIndexRight < formattedImageSeriesRight.length - 1
        ) {
          setImageIndexLeft(parseInt(imageIndexLeft) + 1);
          setImageIndexRight(parseInt(imageIndexRight) + 1);
        }
      } else {
        if (imageIndexLeft > 0 && imageIndexRight > 0) {
          setImageIndexLeft(parseInt(imageIndexLeft) - 1);
          setImageIndexRight(parseInt(imageIndexRight) - 1);
        }
      }
    };
    window.addEventListener("wheel", handleWheel);
    return () => {
      window.removeEventListener("wheel", handleWheel);
    };
  }, [
    imageIndexLeft,
    imageIndexRight,
    formattedImageSeriesLeft,
    formattedImageSeriesRight,
  ]);

  useEffect(() => {
    setLenghtLeft(formattedImageSeriesLeft.length - 1);
    setLenghtRight(formattedImageSeriesRight.length - 1);
  }, [formattedImageSeriesLeft, formattedImageSeriesRight]);

  return (
    <div className="h-full">
      <div className="h-full w-full">
        <div class="flex flex-row w-full">
          <input
            type="range"
            className="form-range w-full h-5"
            value={imageIndexLeft}
            min="0"
            max={lengthLeft}
            step="1"
            onChange={(e) => handleChangeLeft(e)}
          />
          <input
            type="range"
            className="form-range w-full"
            value={imageIndexRight}
            min="0"
            max={lengthRight}
            step="1"
            onChange={(e) => handleChangeRight(e)}
          />
        </div>
        {loading ? (
          <p>Loading...</p>
        ) : splitWindow ? (
          <div className="flex flex-row h-full w-screen">
            <ViewportWrapper
              toolStates={state.tools}
              formattedImageSeries={formattedImageSeriesLeft}
              splitWindow={splitWindow}
              colormap={colormapLeft}
              inverted={inverted}
              opacity={opacity}
              currentFilter={currentFilter}
              imageIndex={imageIndexLeft}
              setImageIndex={setImageIndexLeft}
            />
            <ViewportWrapper
              toolStates={state.tools}
              formattedImageSeries={formattedImageSeriesRight}
              splitWindow={splitWindow}
              colormap={colormapRight}
              inverted={inverted}
              imageIndex={imageIndexRight}
              setImageIndex={setImageIndexRight}
            />
          </div>
        ) : (
          <ReactCompareSlider
            style={{ width: "100%", height: "100%" }}
            itemOne={
              <ViewportWrapper
                toolStates={state.tools}
                formattedImageSeries={formattedImageSeriesLeft}
                splitWindow={splitWindow}
                colormap={colormapLeft}
                inverted={inverted}
                opacity={opacity}
                currentFilter={currentFilter}
                imageIndex={imageIndexLeft}
                setImageIndex={setImageIndexLeft}
              />
            }
            itemTwo={
              <ViewportWrapper
                toolStates={state.tools}
                formattedImageSeries={formattedImageSeriesRight}
                splitWindow={splitWindow}
                colormap={colormapRight}
                inverted={inverted}
                imageIndex={imageIndexRight}
                setImageIndex={setImageIndexRight}
              />
            }
            onlyHandleDraggable
          />
        )}
      </div>
    </div>
  );
}
